import os
import pandas as pd
import range_pb2

def print_attrs(obj):
    for attr in dir(obj):
        if not attr.startswith("__"):  # 过滤魔术方法
            try:
                value = getattr(obj, attr)
                print(f"{attr}: {value}")
            except Exception as e:
                print(f"{attr}: [无法读取，错误：{e}]")

def Base_to_proto_binary(csv_path, base_data):
    # 读取 csv 数据
    df = pd.read_csv(csv_path, encoding='utf-8-sig', header=None, names=["字段", "值"])

    base_data.by_boat = int(df[df["字段"] == "乘船"]["值"].values[0])
    base_data.unuse = int(df[df["字段"] == "无用"]["值"].values[0])
    base_data.world_x = int(df[df["字段"] == "人X"]["值"].values[0])
    base_data.world_y = int(df[df["字段"] == "人Y"]["值"].values[0])
    base_data.scene_x = int(df[df["字段"] == "人X1坐标"]["值"].values[0])
    base_data.scene_y = int(df[df["字段"] == "人Y1坐标"]["值"].values[0])
    base_data.lead_face = int(df[df["字段"] == "人面对方向"]["值"].values[0])
    base_data.boat_x = int(df[df["字段"] == "船X"]["值"].values[0])
    base_data.boat_y = int(df[df["字段"] == "船Y"]["值"].values[0])
    base_data.boat_x1 = int(df[df["字段"] == "船X1"]["值"].values[0])
    base_data.boat_y1 = int(df[df["字段"] == "船Y1"]["值"].values[0])
    base_data.boat_face = int(df[df["字段"] == "船面对方向"]["值"].values[0])
    base_data.guid = df[df["字段"] == "GUID"]["值"].fillna("").values[0]
    base_data.difficulty = int(df[df["字段"] == "难度"]["值"].values[0])
    base_data.standard = int(df[df["字段"] == "标准"]["值"].values[0])
    base_data.fantasy = int(df[df["字段"] == "畅想"]["值"].values[0])
    base_data.special = int(df[df["字段"] == "特殊"]["值"].values[0])
    base_data.single_mode = int(df[df["字段"] == "单通"]["值"].values[0])
    base_data.playthrough = int(df[df["字段"] == "周目"]["值"].values[0])
    base_data.book_count = int(df[df["字段"] == "天书数量"]["值"].values[0])

    # CSV 字段格式：队友, 队友1, 队友2...队友14（值可能为 -1 表示无队友）
    teammate_ids = []
    for i in range(1, 16):
        if i == 1:
            field_name = "队友"
        else:
            field_name = f"队友{i-1}"
        value = int(df[df["字段"] == field_name]["值"].values[0])
        teammate_ids.append(int(value))
    base_data.teammate.extend(teammate_ids)  # 赋值给 repeated int32 字段

    # CSV 字段格式：物品1, 物品个数1...物品399, 物品个数399
    item_ids = []
    item_quantities = []
    for i in range(1, 400):  # 遍历物品1~物品399
        if i == 1:
            item_field = "物品"
            count_field = "物品个数"
        else:
            item_field = f"物品{i-1}"
            count_field = f"物品个数{i-1}"
        item_id = df[df["字段"] == item_field]["值"].values[0]
        item_count = df[df["字段"] == count_field]["值"].values[0]
        item_ids.append(int(item_id))
        item_quantities.append(int(item_count))

    base_data.items.extend(item_ids)        # 赋值物品ID列表
    base_data.item_counts.extend(item_quantities)  # 赋值物品数量列表


def Person_to_proto_binary(csv_path, range_data):

    # 读取 csv 数据
    df = pd.read_csv(csv_path, encoding='utf-8-sig')
    
    for index, row in df.iterrows():
        person = range_data.persons.add()
        person.id = int(row["代号"])
        person.avatar_id = row["头像/战斗代号"]
        person.hp_growth = int(row["生命增长"])
        person.unused = int(row["无用"])
        person.name = row["姓名"]
        person.nickname = row["外号"]
        person.nickname2 = row["外号2"]
        person.gender = int(row["性别"])
        person.level = int(row["等级"])
        person.exp = int(row["经验值"])
        person.hp = int(row["生命"])
        person.max_hp = int(row["生命最大值"])
        person.injury = int(row["受伤程度"])
        person.poison = int(row["中毒程度"])
        person.stamina = int(row["体力"])
        person.weapon = int(row["武器"])
        person.armor = int(row["防具"])
        attack_frames = []
        for i in range(1, 6):
            if i == 1:
                atk_frame = "出招动画帧数"
            else:
                atk_frame = f"出招动画帧数.{i-1}"
            attack_frames.append(int(row[atk_frame]))
        person.attack_frames.extend(attack_frames)
        attack_delays = []
        for i in range(1, 6):
            if i == 1:
                atk_delay = "出招动画延迟"
            else:
                atk_delay = f"出招动画延迟.{i-1}"
            attack_delays.append(int(row[atk_delay]))
        person.attack_delays.extend(attack_delays)
        skill_sound_delays = []
        for i in range(1, 6):
            if i == 1:
                skill_sound_delay = "武功音效延迟"
            else:
                skill_sound_delay = f"武功音效延迟.{i-1}"
            skill_sound_delays.append(int(row[skill_sound_delay]))
        person.skill_sound_delays.extend(skill_sound_delays)
        person.neili_type = int(row["内力性质"])
        person.neili = int(row["内力"])
        person.max_neili = int(row["内力最大值"])
        person.attack = int(row["攻击力"])
        person.speed = int(row["轻功"])
        person.defense = int(row["防御力"])
        person.medical = int(row["医疗"])
        person.poison_skill = int(row["用毒"])
        person.detoxify = int(row["解毒"])
        person.poison_resist = int(row["抗毒"])
        person.xishu_quan = int(row["拳掌"])
        person.xishu_zhi = int(row["指法"])
        person.xishu_jian = int(row["御剑"])
        person.xishu_dao = int(row["耍刀"])
        person.xishu_qi = int(row["特殊兵器"])
        person.xishu_anqi = int(row["暗器技巧"])
        person.wu_chang = int(row["武学常识"])
        person.daode = int(row["品德"])
        person.poison_attack = int(row["攻击带毒"])
        person.zuo_you = int(row["左右互搏"])
        person.reputation = int(row["声望"])
        person.zi_zhi = int(row["资质"])
        person.training_item = int(row["修练物品"])
        person.training_points = int(row["修炼点数"])
        person.product_points = int(row["物品修炼点数"])
        learned_wugong = []
        for i in range(1, 16):
            learned_wugong_field = f"所会武功{i}"
            learned_wugong.append(int(row[learned_wugong_field]))
        person.wugong.extend(learned_wugong)
        learned_wugong_levels = []
        for i in range(1, 16):
            learned_wugong_level_field = f"所会武功{i}等级"
            learned_wugong_levels.append(int(row[learned_wugong_level_field]))
        person.wugong_levels.extend(learned_wugong_levels)
        carryed_items = []
        for i in range(1, 5):
            carryed_item_field = f"携带物品{i}"
            carryed_items.append(int(row[carryed_item_field]))
        person.items.extend(carryed_items)
        carrryed_item_counts = []
        for i in range(1, 5):
            carryed_item_count_field = f"携带物品{i}数量"
            carrryed_item_counts.append(int(row[carryed_item_count_field]))
        person.item_counts.extend(carrryed_item_counts)
        tian_wai = []
        for i in range(1, 5):
            tian_wai_field = f"天赋外功{i}"
            tian_wai.append(int(row[tian_wai_field]))
        person.tian_wai.extend(tian_wai)
        person.tian_nei = int(row["天赋内功"])
        person.tian_qing = int(row["天赋轻功"])
        person.shi_zhan = int(row["实战"])
        person.chu_zhan = int(row["出战"])
        person.xiu_zhan = int(row["休战"])
        person.bing_feng = int(row["冰封程度"])
        person.hp_multiplier = int(row["血量翻倍"])
        person.zhuo_shao = int(row["灼烧程度"])
        person.active_neigong = int(row["主运内功"])
        person.active_qinggong = int(row["主运轻功"])
        person.personal_awaken = int(row["个人觉醒"])
        person.fantasy_class = int(row["畅想分阶"])
        person.ai_mode = int(row["行为模式"])
        person.ai_priority = int(row["优先使用"])
        person.auto_heal = int(row["是否吃药"])
        person.heal_hp_threshold = int(row["生命阈值"])
        person.heal_mp_threshold = int(row["内力阈值"])
        person.heal_stamina_threshold = int(row["体力阈值"])
        person.disable_auto = int(row["禁用自动"])
        person.special_command = int(row["特色指令"])
        anqi = []
        for i in range(1, 4):
            anqi_field = f"暗器{i}"
            anqi.append(int(row[anqi_field]))
        person.hidden_weapons.extend(anqi)
        person.animal_companion = int(row["动物伙伴"])

def Items_to_proto_binary(csv_path, range_data):

    # 读取 csv 数据
    df = pd.read_csv(csv_path, encoding='utf-8-sig')
    
    for index, row in df.iterrows():
        item = range_data.items.add()
        item.id = int(row["代号"])
        item.name = row["物品名"]
        item.description = row["物品说明"]
        item.learn_wugong = int(row["练出武功"])
        item.animation_id = int(row["暗器动画编号"])
        item.user_id = int(row["使用人"])
        item.equip_type = int(row["装备类型"])
        item.show_description = bool(int(row["显示物品说明"]))
        item.type = int(row["类型"])
        item.hp_mod = int(row["加生命"])
        item.max_hp_mod = row["加生命最大值"]
        item.poison_mod = int(row["加中毒解毒"])
        item.stamina_mod = int(row["加体力"])
        item.neili_type_mod = int(row["改变内力性质"])
        item.neili_mod = int(row["加内力"])
        item.max_neili_mod = int(row["加内力最大值"])
        item.attack_mod = int(row["加攻击力"])
        item.speed_mod = int(row["加轻功"])
        item.defense_mod = int(row["加防御力"])
        item.medical_skill_mod = int(row["加医疗"])
        item.poison_skill_mod = int(row["加使毒"])
        item.detox_skill_mod = int(row["加解毒"])
        item.poison_resist_mod = int(row["加抗毒"])
        item.quan_mod = int(row["加拳掌"])
        item.zhi_mod = int(row["加指法"])
        item.jian_mod = int(row["加御剑"])
        item.dao_mod = int(row["加耍刀"])
        item.special_mod = int(row["加特殊兵器"])
        item.anqi_mod = int(row["加暗器技巧"])
        item.wuchang_mod = int(row["加武学常识"])
        item.daode_mod = int(row["加品德"])
        item.attack_count_mod = int(row["加攻击次数"])
        item.attack_poison_mod = int(row["加功夫带毒"])
        item.only_user = int(row["仅修炼人物"])
        item.require_neili_type = int(row["需内力性质"])
        item.require_neili = int(row["需内力"])
        item.require_attack = int(row["需攻击力"])
        item.require_speed = int(row["需轻功"])
        item.require_poison_skill = int(row["需用毒"])
        item.require_medical = int(row["需医疗"])
        item.require_detox = int(row["需解毒"])
        item.require_quan = int(row["需拳掌"])
        item.require_zhi = int(row["需指法"])
        item.require_jian = int(row["需御剑"])
        item.require_dao = int(row["需耍刀"])
        item.require_special = int(row["需特殊兵器"])
        item.require_anqi = int(row["需暗器"])
        item.require_zizhi = int(row["需资质"])
        item.learn_require_exp = int(row["需经验"])
        item.product_exp = int(row["练出物品需经验"])
        item.material_flag = int(row["需材料"])
        craft_items = []
        for i in range(1, 6):
            craft_item = f"练出物品{i}"
            craft_items.append(int(row[craft_item]))
        item.craft_items.extend(craft_items)
        material_counts = []
        for i in range(1, 6):
            material_count = f"需要物品{i}数量"
            material_counts.append(int(row[material_count]))
        item.material_counts.extend(material_counts)
        item.has_effect = bool(int(row["是否特效"]))
        item.equip_exp = int(row["装备经验"])
        item.equip_level = int(row["装备等级"])

def Scene_to_proto_binary(csv_path, range_data):

    # 读取 csv 数据
    df = pd.read_csv(csv_path, encoding='utf-8-sig')
    
    for index, row in df.iterrows():
        scene = range_data.scenes.add()
        scene.id = int(row["代号"])
        scene.name = row["名称"]
        scene.exit_bgm = int(row["出门音乐"])
        scene.enter_bgm = int(row["进门音乐"])
        scene.access_condition = int(row["进入条件"])
        scene.outside_entrance1.x = int(row["外景入口X1"])
        scene.outside_entrance1.y = int(row["外景入口Y1"])
        scene.outside_entrance2.x = int(row["外景入口X2"])
        scene.outside_entrance2.y = int(row["外景入口Y2"])
        scene.position_entering.x = int(row["入口X"])
        scene.position_entering.y = int(row["入口Y"])
        for i in range(1, 4):
            exit = scene.exits.add()
            exit_field_x = f"出口X{i}"
            exit_field_y = f"出口Y{i}"
            exit.x = int(row[exit_field_x])
            exit.y = int(row[exit_field_y])
        scene.target_scene = int(row["跳转场景"])
        for i in range(1, 3):
            target_exit = scene.jump_points.add()
            target_exit.x = int(row[f"跳转口X{i}"])
            target_exit.y = int(row[f"跳转口Y{i}"])
        scene.scene_type = int(row["场景类型"])

def Wugong_to_proto_binary(csv_path, range_data):

    # 读取 csv 数据
    df = pd.read_csv(csv_path, encoding='utf-8-sig')
    
    for index, row in df.iterrows():
        wugong = range_data.wugongs.add()
        wugong.id = int(row["代号"])
        wugong.name = row["名称"]
        wugong.cast_sound = int(row["出招音效"])
        wugong.animation_id = int(row["武功动画&音效"])
        wugong.type = int(row["武功类型"])
        wugong.damage_type = int(row["伤害类型"])
        wugong.attack_pattern = int(row["攻击范围"])
        wugong.neili_cost = int(row["消耗内力点数"])
        wugong.poison_value = int(row["敌人中毒点数"])
        attack_levels = []
        move_range_levels = []
        damage_range_levels = []
        neili_gain_levels = []
        neili_damage_levels = []
        for i in range(1, 11):
            attack_levels.append(int(row[f"{i}级攻击力"]))
            move_range_levels.append(int(row[f"{i}级移动范围"]))
            damage_range_levels.append(int(row[f"{i}级杀伤范围"]))
            neili_gain_levels.append(int(row[f"{i}级加内力"]))
            neili_damage_levels.append(int(row[f"{i}级杀伤内力"]))
        wugong.growth.attack.levels.extend(attack_levels)
        wugong.growth.move_range.levels.extend(move_range_levels)
        wugong.growth.damage_range.levels.extend(damage_range_levels)
        wugong.growth.neili_gain.levels.extend(neili_gain_levels)
        wugong.growth.neili_damage.levels.extend(neili_damage_levels)
        wugong.effects.bing_feng = bool(int(row["冰封系数"]))
        wugong.effects.zhuo_shao = bool(int(row["灼烧系数"]))

def Shop_to_proto_binary(csv_path, range_data):
    # 读取 csv 数据
    df = pd.read_csv(csv_path, encoding='utf-8-sig')
    
    for index, row in df.iterrows():
        shop = range_data.shops.add()
        for i in range(1, 7):
            slot = shop.slots.add()
            slot.item_id = int(row[f"物品{i}"])
            slot.quantity = int(row[f"物品{i}数量"])
            slot.price = int(row[f"物品{i}价格"])


if __name__ == "__main__":
    range_data = range_pb2.Range()
    # 读取Base
    Base_to_proto_binary("base.csv", range_data.base)
    # 读取Person
    Person_to_proto_binary("person.csv", range_data)
    # 读取Items
    Items_to_proto_binary("items.csv", range_data)
    # 读取场景
    Scene_to_proto_binary("scenes.csv", range_data)
    # 读取Wugong
    Wugong_to_proto_binary("wugongs.csv", range_data)
    # 读取Shop
    Shop_to_proto_binary("shops.csv", range_data)

    print("Person size:", len(range_data.persons))
    print("Item size:", len(range_data.items))
    print("Scene size:", len(range_data.scenes))
    print("Wugong size:", len(range_data.wugongs))
    print("Shop size:", len(range_data.shops))
    # print_attrs(range_data)

    # 将数据序列化为二进制文件
    with open("r.bin", "wb") as f:
        f.write(range_data.SerializeToString())